@@ -39,6 +39,7 @@
 #include "ginputstream.h"
 #include "giostream.h"
 #include "gsocketcontrolmessage.h"
+#include "gsocketconnection.h"
 
 #ifdef G_OS_UNIX
 #include "gunixfdmessage.h"
@@ -46,13 +47,16 @@
 #include "gunixcredentialsmessage.h"
 #endif
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
+
 #include "glibintl.h"
-#include "gioalias.h"
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static gchar *
-hexdump (const gchar *data, gsize len, guint indent)
+gchar *
+_g_dbus_hexdump (const gchar *data, gsize len, guint indent)
 {
  guint n, m;
  GString *ret;
@@ -114,6 +118,7 @@ read_with_control_data_free (ReadWithControlData *data)
   g_object_unref (data->socket);
   if (data->cancellable != NULL)
     g_object_unref (data->cancellable);
+  g_object_unref (data->simple);
   g_free (data);
 }
 
@@ -345,11 +350,21 @@ _g_dbus_shared_thread_unref (void)
 struct GDBusWorker
 {
   volatile gint                       ref_count;
+
   gboolean                            stopped;
+
+  /* TODO: frozen (e.g. G_DBUS_CONNECTION_FLAGS_DELAY_MESSAGE_PROCESSING) currently
+   * only affects messages received from the other peer (since GDBusServer is the
+   * only user) - we might want it to affect messages sent to the other peer too?
+   */
+  gboolean                            frozen;
+  GQueue                             *received_messages_while_frozen;
+
   GIOStream                          *stream;
   GDBusCapabilityFlags                capabilities;
   GCancellable                       *cancellable;
   GDBusWorkerMessageReceivedCallback  message_received_callback;
+  GDBusWorkerMessageAboutToBeSentCallback message_about_to_be_sent_callback;
   GDBusWorkerDisconnectedCallback     disconnected_callback;
   gpointer                            user_data;
 
@@ -372,8 +387,17 @@ struct GDBusWorker
   GMutex                             *write_lock;
   GQueue                             *write_queue;
   gboolean                            write_is_pending;
+  guint64                             write_num_messages_written;
+  GList                              *write_pending_flushes;
 };
 
+typedef struct
+{
+  GMutex *mutex;
+  GCond *cond;
+  guint64 number_to_wait_for;
+} FlushData;
+
 struct _MessageToWriteData ;
 typedef struct _MessageToWriteData MessageToWriteData;
 
@@ -391,6 +415,8 @@ _g_dbus_worker_unref (GDBusWorker *worker)
 {
   if (g_atomic_int_dec_and_test (&worker->ref_count))
     {
+      g_assert (worker->write_pending_flushes == NULL);
+
       _g_dbus_shared_thread_unref ();
 
       g_object_unref (worker->stream);
@@ -400,11 +426,13 @@ _g_dbus_worker_unref (GDBusWorker *worker)
       if (worker->read_fd_list != NULL)
         g_object_unref (worker->read_fd_list);
 
+      g_queue_foreach (worker->received_messages_while_frozen, (GFunc) g_object_unref, NULL);
+      g_queue_free (worker->received_messages_while_frozen);
+
       g_mutex_free (worker->write_lock);
-      g_queue_foreach (worker->write_queue,
-                       (GFunc) message_to_write_data_free,
-                       NULL);
+      g_queue_foreach (worker->write_queue, (GFunc) message_to_write_data_free, NULL);
       g_queue_free (worker->write_queue);
+
       g_free (worker);
     }
 }
@@ -419,13 +447,84 @@ _g_dbus_worker_emit_disconnected (GDBusWorker  *worker,
 }
 
 static void
-_g_dbus_worker_emit_message (GDBusWorker  *worker,
-                             GDBusMessage *message)
+_g_dbus_worker_emit_message_received (GDBusWorker  *worker,
+                                      GDBusMessage *message)
 {
   if (!worker->stopped)
     worker->message_received_callback (worker, message, worker->user_data);
 }
 
+static gboolean
+_g_dbus_worker_emit_message_about_to_be_sent (GDBusWorker  *worker,
+                                              GDBusMessage *message)
+{
+  gboolean ret;
+  ret = FALSE;
+  if (!worker->stopped)
+    ret = worker->message_about_to_be_sent_callback (worker, message, worker->user_data);
+  return ret;
+}
+
+/* can only be called from private thread with read-lock held - takes ownership of @message */
+static void
+_g_dbus_worker_queue_or_deliver_received_message (GDBusWorker  *worker,
+                                                  GDBusMessage *message)
+{
+  if (worker->frozen || g_queue_get_length (worker->received_messages_while_frozen) > 0)
+    {
+      /* queue up */
+      g_queue_push_tail (worker->received_messages_while_frozen, message);
+    }
+  else
+    {
+      /* not frozen, nor anything in queue */
+      _g_dbus_worker_emit_message_received (worker, message);
+      g_object_unref (message);
+    }
+}
+
+/* called in private thread shared by all GDBusConnection instances (without read-lock held) */
+static gboolean
+unfreeze_in_idle_cb (gpointer user_data)
+{
+  GDBusWorker *worker = user_data;
+  GDBusMessage *message;
+
+  g_mutex_lock (worker->read_lock);
+  if (worker->frozen)
+    {
+      while ((message = g_queue_pop_head (worker->received_messages_while_frozen)) != NULL)
+        {
+          _g_dbus_worker_emit_message_received (worker, message);
+          g_object_unref (message);
+        }
+      worker->frozen = FALSE;
+    }
+  else
+    {
+      g_assert (g_queue_get_length (worker->received_messages_while_frozen) == 0);
+    }
+  g_mutex_unlock (worker->read_lock);
+  return FALSE;
+}
+
+/* can be called from any thread */
+void
+_g_dbus_worker_unfreeze (GDBusWorker *worker)
+{
+  GSource *idle_source;
+  idle_source = g_idle_source_new ();
+  g_source_set_priority (idle_source, G_PRIORITY_DEFAULT);
+  g_source_set_callback (idle_source,
+                         unfreeze_in_idle_cb,
+                         _g_dbus_worker_ref (worker),
+                         (GDestroyNotify) _g_dbus_worker_unref);
+  g_source_attach (idle_source, shared_thread_data->context);
+  g_source_unref (idle_source);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void _g_dbus_worker_do_read_unlocked (GDBusWorker *worker);
 
 /* called in private thread shared by all GDBusConnection instances (without read-lock held) */
@@ -583,20 +682,33 @@ _g_dbus_worker_do_read_cb (GInputStream  *input_stream,
                                                   &error);
           if (message == NULL)
             {
+              gchar *s;
+              s = _g_dbus_hexdump (worker->read_buffer, worker->read_buffer_cur_size, 2);
+              g_warning ("Error decoding D-Bus message of %" G_GSIZE_FORMAT " bytes\n"
+                         "The error is: %s\n"
+                         "The payload is as follows:\n"
+                         "%s\n",
+                         worker->read_buffer_cur_size,
+                         error->message,
+                         s);
+              g_free (s);
               _g_dbus_worker_emit_disconnected (worker, FALSE, error);
               g_error_free (error);
               goto out;
             }
 
+#ifdef G_OS_UNIX
           if (worker->read_fd_list != NULL)
             {
               g_dbus_message_set_unix_fd_list (message, worker->read_fd_list);
               worker->read_fd_list = NULL;
             }
+#endif
 
           if (G_UNLIKELY (_g_dbus_debug_message ()))
             {
               gchar *s;
+              _g_dbus_debug_print_lock ();
               g_print ("========================================================================\n"
                        "GDBus-debug:Message:\n"
                        "  <<<< RECEIVED D-Bus message (%" G_GSIZE_FORMAT " bytes)\n",
@@ -604,14 +716,17 @@ _g_dbus_worker_do_read_cb (GInputStream  *input_stream,
               s = g_dbus_message_print (message, 2);
               g_print ("%s", s);
               g_free (s);
-              s = hexdump (worker->read_buffer, worker->read_buffer_cur_size, 2);
-              g_print ("%s\n", s);
-              g_free (s);
+              if (G_UNLIKELY (_g_dbus_debug_payload ()))
+                {
+                  s = _g_dbus_hexdump (worker->read_buffer, worker->read_buffer_cur_size, 2);
+                  g_print ("%s\n", s);
+                  g_free (s);
+                }
+              _g_dbus_debug_print_unlock ();
             }
 
           /* yay, got a message, go deliver it */
-          _g_dbus_worker_emit_message (worker, message);
-          g_object_unref (message);
+          _g_dbus_worker_queue_or_deliver_received_message (worker, message);
 
           /* start reading another message! */
           worker->read_buffer_bytes_wanted = 0;
@@ -703,13 +818,15 @@ message_to_write_data_free (MessageToWriteData *data)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-/* called in private thread shared by all GDBusConnection instances (with write-lock held) */
+/* called in private thread shared by all GDBusConnection instances (without write-lock held) */
 static gboolean
 write_message (GDBusWorker         *worker,
                MessageToWriteData  *data,
                GError             **error)
 {
   gboolean ret;
+  GList *l;
+  GList *ll;
 
   g_return_val_if_fail (data->blob_size > 16, FALSE);
 
@@ -803,9 +920,28 @@ write_message (GDBusWorker         *worker,
 
   ret = TRUE;
 
+  /* wake up pending flushes */
+  g_mutex_lock (worker->write_lock);
+  worker->write_num_messages_written += 1;
+  for (l = worker->write_pending_flushes; l != NULL; l = ll)
+    {
+      FlushData *f = l->data;
+      ll = l->next;
+
+      if (f->number_to_wait_for == worker->write_num_messages_written)
+        {
+          g_mutex_lock (f->mutex);
+          g_cond_signal (f->cond);
+          g_mutex_unlock (f->mutex);
+          worker->write_pending_flushes = g_list_delete_link (worker->write_pending_flushes, l);
+        }
+    }
+  g_mutex_unlock (worker->write_lock);
+
   if (G_UNLIKELY (_g_dbus_debug_message ()))
     {
       gchar *s;
+      _g_dbus_debug_print_lock ();
       g_print ("========================================================================\n"
                "GDBus-debug:Message:\n"
                "  >>>> SENT D-Bus message (%" G_GSIZE_FORMAT " bytes)\n",
@@ -813,9 +949,13 @@ write_message (GDBusWorker         *worker,
       s = g_dbus_message_print (data->message, 2);
       g_print ("%s", s);
       g_free (s);
-      s = hexdump (data->blob, data->blob_size, 2);
-      g_print ("%s\n", s);
-      g_free (s);
+      if (G_UNLIKELY (_g_dbus_debug_payload ()))
+        {
+          s = _g_dbus_hexdump (data->blob, data->blob_size, 2);
+          g_print ("%s\n", s);
+          g_free (s);
+        }
+      _g_dbus_debug_print_unlock ();
     }
 
  out:
@@ -831,29 +971,39 @@ write_message_in_idle_cb (gpointer user_data)
   GDBusWorker *worker = user_data;
   gboolean more_writes_are_pending;
   MessageToWriteData *data;
+  gboolean message_was_dropped;
   GError *error;
 
   g_mutex_lock (worker->write_lock);
-
   data = g_queue_pop_head (worker->write_queue);
   g_assert (data != NULL);
-
-  error = NULL;
-  if (!write_message (worker,
-                      data,
-                      &error))
-    {
-      /* TODO: handle */
-      _g_dbus_worker_emit_disconnected (worker, TRUE, error);
-      g_error_free (error);
-    }
-  message_to_write_data_free (data);
-
   more_writes_are_pending = (g_queue_get_length (worker->write_queue) > 0);
-
   worker->write_is_pending = more_writes_are_pending;
   g_mutex_unlock (worker->write_lock);
 
+  /* Note that write_lock is only used for protecting the @write_queue
+   * and @write_is_pending fields of the GDBusWorker struct ... which we
+   * need to modify from arbitrary threads in _g_dbus_worker_send_message().
+   *
+   * Therefore, it's fine to drop it here when calling back into user
+   * code and then writing the message out onto the GIOStream since this
+   * function only runs on the worker thread.
+   */
+  message_was_dropped = _g_dbus_worker_emit_message_about_to_be_sent (worker, data->message);
+  if (G_LIKELY (!message_was_dropped))
+    {
+      error = NULL;
+      if (!write_message (worker,
+                          data,
+                          &error))
+        {
+          /* TODO: handle */
+          _g_dbus_worker_emit_disconnected (worker, TRUE, error);
+          g_error_free (error);
+        }
+    }
+  message_to_write_data_free (data);
+
   return more_writes_are_pending;
 }
 
@@ -911,16 +1061,19 @@ _g_dbus_worker_thread_begin_func (gpointer user_data)
 }
 
 GDBusWorker *
-_g_dbus_worker_new (GIOStream                          *stream,
-                    GDBusCapabilityFlags                capabilities,
-                    GDBusWorkerMessageReceivedCallback  message_received_callback,
-                    GDBusWorkerDisconnectedCallback     disconnected_callback,
-                    gpointer                            user_data)
+_g_dbus_worker_new (GIOStream                              *stream,
+                    GDBusCapabilityFlags                    capabilities,
+                    gboolean                                initially_frozen,
+                    GDBusWorkerMessageReceivedCallback      message_received_callback,
+                    GDBusWorkerMessageAboutToBeSentCallback message_about_to_be_sent_callback,
+                    GDBusWorkerDisconnectedCallback         disconnected_callback,
+                    gpointer                                user_data)
 {
   GDBusWorker *worker;
 
   g_return_val_if_fail (G_IS_IO_STREAM (stream), NULL);
   g_return_val_if_fail (message_received_callback != NULL, NULL);
+  g_return_val_if_fail (message_about_to_be_sent_callback != NULL, NULL);
   g_return_val_if_fail (disconnected_callback != NULL, NULL);
 
   worker = g_new0 (GDBusWorker, 1);
@@ -928,12 +1081,16 @@ _g_dbus_worker_new (GIOStream                          *stream,
 
   worker->read_lock = g_mutex_new ();
   worker->message_received_callback = message_received_callback;
+  worker->message_about_to_be_sent_callback = message_about_to_be_sent_callback;
   worker->disconnected_callback = disconnected_callback;
   worker->user_data = user_data;
   worker->stream = g_object_ref (stream);
   worker->capabilities = capabilities;
   worker->cancellable = g_cancellable_new ();
 
+  worker->frozen = initially_frozen;
+  worker->received_messages_while_frozen = g_queue_new ();
+
   worker->write_lock = g_mutex_new ();
   worker->write_queue = g_queue_new ();
 
@@ -945,6 +1102,8 @@ _g_dbus_worker_new (GIOStream                          *stream,
   return worker;
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
 /* This can be called from any thread - frees worker - guarantees no callbacks
  * will ever be issued again
  */
@@ -965,9 +1124,63 @@ _g_dbus_worker_stop (GDBusWorker *worker)
   _g_dbus_worker_unref (worker);
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+/* can be called from any thread (except the worker thread) - blocks
+ * calling thread until all queued outgoing messages are written and
+ * the transport has been flushed
+ */
+gboolean
+_g_dbus_worker_flush_sync (GDBusWorker    *worker,
+                           GCancellable   *cancellable,
+                           GError        **error)
+{
+  gboolean ret;
+  FlushData *data;
+
+  data = NULL;
+
+  /* if the queue is empty, there's nothing to wait for */
+  g_mutex_lock (worker->write_lock);
+  if (g_queue_get_length (worker->write_queue) > 0)
+    {
+      data = g_new0 (FlushData, 1);
+      data->mutex = g_mutex_new ();
+      data->cond = g_cond_new ();
+      data->number_to_wait_for = worker->write_num_messages_written + g_queue_get_length (worker->write_queue);
+      g_mutex_lock (data->mutex);
+      worker->write_pending_flushes = g_list_prepend (worker->write_pending_flushes, data);
+    }
+  g_mutex_unlock (worker->write_lock);
+
+  if (data != NULL)
+    {
+      g_cond_wait (data->cond, data->mutex);
+      g_mutex_unlock (data->mutex);
+
+      /* note:the element is removed from worker->write_pending_flushes in write_message() */
+      g_cond_free (data->cond);
+      g_mutex_free (data->mutex);
+      g_free (data);
+    }
+
+  ret = g_output_stream_flush (g_io_stream_get_output_stream (worker->stream),
+                               cancellable,
+                               error);
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 #define G_DBUS_DEBUG_AUTHENTICATION (1<<0)
 #define G_DBUS_DEBUG_MESSAGE        (1<<1)
-#define G_DBUS_DEBUG_ALL            0xffffffff
+#define G_DBUS_DEBUG_PAYLOAD        (1<<2)
+#define G_DBUS_DEBUG_CALL           (1<<3)
+#define G_DBUS_DEBUG_SIGNAL         (1<<4)
+#define G_DBUS_DEBUG_INCOMING       (1<<5)
+#define G_DBUS_DEBUG_EMISSION       (1<<6)
+#define G_DBUS_DEBUG_ADDRESS        (1<<7)
+
 static gint _gdbus_debug_flags = 0;
 
 gboolean
@@ -984,6 +1197,62 @@ _g_dbus_debug_message (void)
   return (_gdbus_debug_flags & G_DBUS_DEBUG_MESSAGE) != 0;
 }
 
+gboolean
+_g_dbus_debug_payload (void)
+{
+  _g_dbus_initialize ();
+  return (_gdbus_debug_flags & G_DBUS_DEBUG_PAYLOAD) != 0;
+}
+
+gboolean
+_g_dbus_debug_call (void)
+{
+  _g_dbus_initialize ();
+  return (_gdbus_debug_flags & G_DBUS_DEBUG_CALL) != 0;
+}
+
+gboolean
+_g_dbus_debug_signal (void)
+{
+  _g_dbus_initialize ();
+  return (_gdbus_debug_flags & G_DBUS_DEBUG_SIGNAL) != 0;
+}
+
+gboolean
+_g_dbus_debug_incoming (void)
+{
+  _g_dbus_initialize ();
+  return (_gdbus_debug_flags & G_DBUS_DEBUG_INCOMING) != 0;
+}
+
+gboolean
+_g_dbus_debug_emission (void)
+{
+  _g_dbus_initialize ();
+  return (_gdbus_debug_flags & G_DBUS_DEBUG_EMISSION) != 0;
+}
+
+gboolean
+_g_dbus_debug_address (void)
+{
+  _g_dbus_initialize ();
+  return (_gdbus_debug_flags & G_DBUS_DEBUG_ADDRESS) != 0;
+}
+
+G_LOCK_DEFINE_STATIC (print_lock);
+
+void
+_g_dbus_debug_print_lock (void)
+{
+  G_LOCK (print_lock);
+}
+
+void
+_g_dbus_debug_print_unlock (void)
+{
+  G_UNLOCK (print_lock);
+}
+
 /*
  * _g_dbus_initialize:
  *
@@ -1007,19 +1276,20 @@ _g_dbus_initialize (void)
       debug = g_getenv ("G_DBUS_DEBUG");
       if (debug != NULL)
         {
-          gchar **tokens;
-          guint n;
-          tokens = g_strsplit (debug, ",", 0);
-          for (n = 0; tokens[n] != NULL; n++)
-            {
-              if (g_strcmp0 (tokens[n], "authentication") == 0)
-                _gdbus_debug_flags |= G_DBUS_DEBUG_AUTHENTICATION;
-              else if (g_strcmp0 (tokens[n], "message") == 0)
-                _gdbus_debug_flags |= G_DBUS_DEBUG_MESSAGE;
-              else if (g_strcmp0 (tokens[n], "all") == 0)
-                _gdbus_debug_flags |= G_DBUS_DEBUG_ALL;
-            }
-          g_strfreev (tokens);
+          const GDebugKey keys[] = {
+            { "authentication", G_DBUS_DEBUG_AUTHENTICATION },
+            { "message",        G_DBUS_DEBUG_MESSAGE        },
+            { "payload",        G_DBUS_DEBUG_PAYLOAD        },
+            { "call",           G_DBUS_DEBUG_CALL           },
+            { "signal",         G_DBUS_DEBUG_SIGNAL         },
+            { "incoming",       G_DBUS_DEBUG_INCOMING       },
+            { "emission",       G_DBUS_DEBUG_EMISSION       },
+            { "address",        G_DBUS_DEBUG_ADDRESS        }
+          };
+
+          _gdbus_debug_flags = g_parse_debug_string (debug, keys, G_N_ELEMENTS (keys));
+          if (_gdbus_debug_flags & G_DBUS_DEBUG_PAYLOAD)
+            _gdbus_debug_flags |= G_DBUS_DEBUG_MESSAGE;
         }
 
       g_once_init_leave (&initialized, 1);
@@ -1028,26 +1298,138 @@ _g_dbus_initialize (void)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-gchar *
-_g_dbus_compute_complete_signature (GDBusArgInfo **args,
-                                    gboolean       include_parentheses)
+GVariantType *
+_g_dbus_compute_complete_signature (GDBusArgInfo **args)
 {
-  GString *s;
+  const GVariantType *arg_types[256];
   guint n;
 
-  if (include_parentheses)
-    s = g_string_new ("(");
-  else
-    s = g_string_new ("");
-  if (args != NULL)
+  if (args)
     for (n = 0; args[n] != NULL; n++)
-      g_string_append (s, args[n]->signature);
+      {
+        /* DBus places a hard limit of 255 on signature length.
+         * therefore number of args must be less than 256.
+         */
+        g_assert (n < 256);
 
-  if (include_parentheses)
-    g_string_append_c (s, ')');
+        arg_types[n] = G_VARIANT_TYPE (args[n]->signature);
 
-  return g_string_free (s, FALSE);
+        if G_UNLIKELY (arg_types[n] == NULL)
+          return NULL;
+      }
+  else
+    n = 0;
+
+  return g_variant_type_new_tuple (arg_types, n);
 }
 
-#define __G_DBUS_PRIVATE_C__
-#include "gioaliasdef.c"
+/* ---------------------------------------------------------------------------------------------------- */
+
+#ifdef G_OS_WIN32
+
+extern BOOL WINAPI ConvertSidToStringSidA (PSID Sid, LPSTR *StringSid);
+
+gchar *
+_g_dbus_win32_get_user_sid (void)
+{
+  HANDLE h;
+  TOKEN_USER *user;
+  DWORD token_information_len;
+  PSID psid;
+  gchar *sid;
+  gchar *ret;
+
+  ret = NULL;
+  user = NULL;
+  h = INVALID_HANDLE_VALUE;
+
+  if (!OpenProcessToken (GetCurrentProcess (), TOKEN_QUERY, &h))
+    {
+      g_warning ("OpenProcessToken failed with error code %d", (gint) GetLastError ());
+      goto out;
+    }
+
+  /* Get length of buffer */
+  token_information_len = 0;
+  if (!GetTokenInformation (h, TokenUser, NULL, 0, &token_information_len))
+    {
+      if (GetLastError () != ERROR_INSUFFICIENT_BUFFER)
+        {
+          g_warning ("GetTokenInformation() failed with error code %d", (gint) GetLastError ());
+          goto out;
+        }
+    }
+  user = g_malloc (token_information_len);
+  if (!GetTokenInformation (h, TokenUser, user, token_information_len, &token_information_len))
+    {
+      g_warning ("GetTokenInformation() failed with error code %d", (gint) GetLastError ());
+      goto out;
+    }
+
+  psid = user->User.Sid;
+  if (!IsValidSid (psid))
+    {
+      g_warning ("Invalid SID");
+      goto out;
+    }
+
+  if (!ConvertSidToStringSidA (psid, &sid))
+    {
+      g_warning ("Invalid SID");
+      goto out;
+    }
+
+  ret = g_strdup (sid);
+  LocalFree (sid);
+
+out:
+  g_free (user);
+  if (h != INVALID_HANDLE_VALUE)
+    CloseHandle (h);
+  return ret;
+}
+#endif
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+gchar *
+_g_dbus_get_machine_id (GError **error)
+{
+  gchar *ret;
+  /* TODO: use PACKAGE_LOCALSTATEDIR ? */
+  ret = NULL;
+  if (!g_file_get_contents ("/var/lib/dbus/machine-id",
+                            &ret,
+                            NULL,
+                            error))
+    {
+      g_prefix_error (error, _("Unable to load /var/lib/dbus/machine-id: "));
+    }
+  else
+    {
+      /* TODO: validate value */
+      g_strstrip (ret);
+    }
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+gchar *
+_g_dbus_enum_to_string (GType enum_type, gint value)
+{
+  gchar *ret;
+  GEnumClass *klass;
+  GEnumValue *enum_value;
+
+  klass = g_type_class_ref (enum_type);
+  enum_value = g_enum_get_value (klass, value);
+  if (enum_value != NULL)
+    ret = g_strdup (enum_value->value_nick);
+  else
+    ret = g_strdup_printf ("unknown (value %d)", value);
+  g_type_class_unref (klass);
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
