@@ -92,6 +92,7 @@ struct _GDBusServer
 
   GSocketListener *listener;
   gboolean is_using_listener;
+  gulong run_signal_handler_id;
 
   /* The result of g_main_context_get_thread_default() when the object
    * was created (the GObject _init() function) - this is used for delivery
@@ -158,6 +159,9 @@ g_dbus_server_finalize (GObject *object)
   if (server->authentication_observer != NULL)
     g_object_unref (server->authentication_observer);
 
+  if (server->run_signal_handler_id > 0)
+    g_signal_handler_disconnect (server->listener, server->run_signal_handler_id);
+
   if (server->listener != NULL)
     g_object_unref (server->listener);
 
@@ -349,14 +353,14 @@ g_dbus_server_class_init (GDBusServerClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVE,
-                                   g_param_spec_string ("active",
-                                                        P_("Active"),
-                                                        P_("Whether the server is currently active"),
-                                                        NULL,
-                                                        G_PARAM_READABLE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_BLURB |
-                                                        G_PARAM_STATIC_NICK));
+                                   g_param_spec_boolean ("active",
+                                                         P_("Active"),
+                                                         P_("Whether the server is currently active"),
+                                                         FALSE,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_BLURB |
+                                                         G_PARAM_STATIC_NICK));
 
   /**
    * GDBusServer:authentication-observer:
@@ -489,10 +493,10 @@ g_dbus_server_new_sync (const gchar        *address,
     {
       /* Right now we don't have any transport not using the listener... */
       g_assert (server->is_using_listener);
-      g_signal_connect (G_SOCKET_SERVICE (server->listener),
-                        "run",
-                        G_CALLBACK (on_run),
-                        server);
+      server->run_signal_handler_id = g_signal_connect (G_SOCKET_SERVICE (server->listener),
+                                                        "run",
+                                                        G_CALLBACK (on_run),
+                                                        server);
     }
 
   return server;
@@ -605,6 +609,9 @@ g_dbus_server_stop (GDBusServer *server)
     return;
   /* Right now we don't have any transport not using the listener... */
   g_assert (server->is_using_listener);
+  g_assert (server->run_signal_handler_id > 0);
+  g_signal_handler_disconnect (server->listener, server->run_signal_handler_id);
+  server->run_signal_handler_id = 0;
   g_socket_service_stop (G_SOCKET_SERVICE (server->listener));
   server->active = FALSE;
   g_object_notify (G_OBJECT (server), "active");
