@@ -30,7 +30,6 @@
 #include <gio/gsimpleasyncresult.h>
 #include <gio/gasyncresult.h>
 
-#include "gioalias.h"
 
 G_DEFINE_TYPE (GIOStream, g_io_stream, G_TYPE_OBJECT);
 
@@ -223,7 +222,8 @@ g_io_stream_is_closed (GIOStream *stream)
  * Gets the input stream for this object. This is used
  * for reading.
  *
- * Returns: a #GInputStream, owned by the #GIOStream. Do not free.
+ * Returns: (transfer none): a #GInputStream, owned by the #GIOStream.
+ * Do not free.
  *
  * Since: 2.22
  */
@@ -246,7 +246,8 @@ g_io_stream_get_input_stream (GIOStream *stream)
  * Gets the output stream for this object. This is used for
  * writing.
  *
- * Returns: a #GOutputStream, owned by the #GIOStream. Do not free.
+ * Returns: (transfer none): a #GOutputStream, owned by the #GIOStream.
+ * Do not free.
  *
  * Since: 2.22
  */
@@ -609,6 +610,3 @@ g_io_stream_real_close_finish (GIOStream     *stream,
 		  g_io_stream_real_close_async);
   return TRUE;
 }
-
-#define __G_IO_STREAM_C__
-#include "gioaliasdef.c"
