@@ -29,9 +29,19 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+/* for g_unlink() */
+#include <glib/gstdio.h>
+
 #include <gio/gunixsocketaddress.h>
 #include <gio/gunixfdlist.h>
 
+/* for struct ucred */
+#ifdef __linux__
+#define __USE_GNU
+#include <sys/types.h>
+#include <sys/socket.h>
+#endif
+
 #include "gdbus-tests.h"
 
 
@@ -68,6 +78,7 @@ static const gchar *test_interface_introspection_xml =
   "      <arg type='s' name='response' direction='out'/>"
   "    </method>"
   "    <method name='EmitSignal'/>"
+  "    <method name='EmitSignalWithNameSet'/>"
   "    <method name='OpenFile'>"
   "      <arg type='s' name='path' direction='in'/>"
   "    </method>"
@@ -77,7 +88,7 @@ static const gchar *test_interface_introspection_xml =
   "    <property type='s' name='PeerProperty' access='read'/>"
   "  </interface>"
   "</node>";
-static const GDBusInterfaceInfo *test_interface_introspection_data = NULL;
+static GDBusInterfaceInfo *test_interface_introspection_data = NULL;
 
 static void
 test_interface_method_call (GDBusConnection       *connection,
@@ -122,6 +133,25 @@ test_interface_method_call (GDBusConnection       *connection,
                                      NULL,
                                      &error);
       g_assert_no_error (error);
+      g_dbus_method_invocation_return_value (invocation, NULL);
+    }
+  else if (g_strcmp0 (method_name, "EmitSignalWithNameSet") == 0)
+    {
+      GError *error;
+      gboolean ret;
+      GDBusMessage *message;
+
+      message = g_dbus_message_new_signal ("/org/gtk/GDBus/PeerTestObject",
+                                           "org.gtk.GDBus.PeerTestInterface",
+                                           "PeerSignalWithNameSet");
+      g_dbus_message_set_sender (message, ":1.42");
+
+      error = NULL;
+      ret = g_dbus_connection_send_message (connection, message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, &error);
+      g_assert_no_error (error);
+      g_assert (ret);
+      g_object_unref (message);
+
       g_dbus_method_invocation_return_value (invocation, NULL);
     }
   else if (g_strcmp0 (method_name, "OpenFile") == 0)
@@ -151,6 +181,7 @@ test_interface_method_call (GDBusConnection       *connection,
       error = NULL;
       g_dbus_connection_send_message (connection,
                                       reply,
+                                      G_DBUS_SEND_MESSAGE_FLAGS_NONE,
                                       NULL, /* out_serial */
                                       &error);
       g_assert_no_error (error);
@@ -207,6 +238,22 @@ on_proxy_signal_received (GDBusProxy *proxy,
   g_main_loop_quit (loop);
 }
 
+static void
+on_proxy_signal_received_with_name_set (GDBusProxy *proxy,
+                                        gchar      *sender_name,
+                                        gchar      *signal_name,
+                                        GVariant   *parameters,
+                                        gpointer    user_data)
+{
+  PeerData *data = user_data;
+
+  data->signal_received = TRUE;
+
+  g_assert_cmpstr (sender_name, ==, ":1.42");
+  g_assert_cmpstr (signal_name, ==, "PeerSignalWithNameSet");
+  g_main_loop_quit (loop);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean
@@ -474,6 +521,7 @@ test_peer (void)
   GVariant *result;
   const gchar *s;
   GThread *service_thread;
+  gulong signal_handler_id;
 
   memset (&data, '\0', sizeof (PeerData));
   data.current_connections = g_ptr_array_new_with_free_func (g_object_unref);
@@ -555,10 +603,10 @@ test_peer (void)
   g_assert_cmpint (data.num_method_calls, ==, 1);
 
   /* make the other peer emit a signal - catch it */
-  g_signal_connect (proxy,
-                    "g-signal",
-                    G_CALLBACK (on_proxy_signal_received),
-                    &data);
+  signal_handler_id = g_signal_connect (proxy,
+                                        "g-signal",
+                                        G_CALLBACK (on_proxy_signal_received),
+                                        &data);
   g_assert (!data.signal_received);
   g_dbus_proxy_call (proxy,
                      "EmitSignal",
@@ -571,6 +619,31 @@ test_peer (void)
   g_main_loop_run (loop);
   g_assert (data.signal_received);
   g_assert_cmpint (data.num_method_calls, ==, 2);
+  g_signal_handler_disconnect (proxy, signal_handler_id);
+
+  /* Also ensure that messages with the sender header-field set gets
+   * delivered to the proxy - note that this doesn't really make sense
+   * e.g. names are meaning-less in a peer-to-peer case... but we
+   * support it because it makes sense in certain bridging
+   * applications - see e.g. #623815.
+   */
+  signal_handler_id = g_signal_connect (proxy,
+                                        "g-signal",
+                                        G_CALLBACK (on_proxy_signal_received_with_name_set),
+                                        &data);
+  data.signal_received = FALSE;
+  g_dbus_proxy_call (proxy,
+                     "EmitSignalWithNameSet",
+                     NULL,  /* no arguments */
+                     G_DBUS_CALL_FLAGS_NONE,
+                     -1,
+                     NULL,  /* GCancellable */
+                     NULL,  /* GAsyncReadyCallback - we don't care about the result */
+                     NULL); /* user_data */
+  g_main_loop_run (loop);
+  g_assert (data.signal_received);
+  g_assert_cmpint (data.num_method_calls, ==, 3);
+  g_signal_handler_disconnect (proxy, signal_handler_id);
 
   /* check for UNIX fd passing */
 #ifdef G_OS_UNIX
@@ -592,6 +665,7 @@ test_peer (void)
     error = NULL;
     method_reply_message = g_dbus_connection_send_message_with_reply_sync (c,
                                                                            method_call_message,
+                                                                           G_DBUS_SEND_MESSAGE_FLAGS_NONE,
                                                                            -1,
                                                                            NULL, /* out_serial */
                                                                            NULL, /* cancellable */
@@ -636,6 +710,35 @@ test_peer (void)
   g_error_free (error);
 #endif /* G_OS_UNIX */
 
+  /* Check that g_socket_get_credentials() work - this really should
+   * be in a GSocket-specific test suite but no such test suite exists
+   * right now.
+   */
+  {
+    GSocket *socket;
+    GCredentials *credentials;
+    socket = g_socket_connection_get_socket (G_SOCKET_CONNECTION (g_dbus_connection_get_stream (c)));
+    g_assert (G_IS_SOCKET (socket));
+    error = NULL;
+    credentials = g_socket_get_credentials (socket, &error);
+#ifdef __linux__
+    {
+      struct ucred *native_creds;
+      g_assert_no_error (error);
+      g_assert (G_IS_CREDENTIALS (credentials));
+      native_creds = g_credentials_get_native (credentials, G_CREDENTIALS_TYPE_LINUX_UCRED);
+      g_assert (native_creds != NULL);
+      g_assert (native_creds->uid == getuid ());
+      g_assert (native_creds->gid == getgid ());
+      g_assert (native_creds->pid == getpid ());
+    }
+    g_object_unref (credentials);
+#else
+    g_assert_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED);
+    g_assert (credentials == NULL);
+#endif
+  }
+
 
   /* bring up a connection - don't accept it - this should fail
    */
@@ -708,7 +811,7 @@ test_peer (void)
   g_variant_get (result, "(&s)", &s);
   g_assert_cmpstr (s, ==, "You greeted me with 'Hey Again Peer!'.");
   g_variant_unref (result);
-  g_assert_cmpint (data.num_method_calls, ==, 4);
+  g_assert_cmpint (data.num_method_calls, ==, 5);
 
 #if 0
   /* TODO: THIS TEST DOESN'T WORK YET */
@@ -732,6 +835,391 @@ test_peer (void)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  GDBusServer *server;
+  GMainContext *context;
+  GMainLoop *loop;
+
+  GList *connections;
+} DmpData;
+
+static void
+dmp_data_free (DmpData *data)
+{
+  g_main_loop_unref (data->loop);
+  g_main_context_unref (data->context);
+  g_object_unref (data->server);
+  g_list_foreach (data->connections, (GFunc) g_object_unref, NULL);
+  g_list_free (data->connections);
+  g_free (data);
+}
+
+static void
+dmp_on_method_call (GDBusConnection       *connection,
+                    const gchar           *sender,
+                    const gchar           *object_path,
+                    const gchar           *interface_name,
+                    const gchar           *method_name,
+                    GVariant              *parameters,
+                    GDBusMethodInvocation *invocation,
+                    gpointer               user_data)
+{
+  //DmpData *data = user_data;
+  gint32 first;
+  gint32 second;
+  g_variant_get (parameters,
+                 "(ii)",
+                 &first,
+                 &second);
+  g_dbus_method_invocation_return_value (invocation,
+                                         g_variant_new ("(i)", first + second));
+}
+
+static const GDBusInterfaceVTable dmp_interface_vtable =
+{
+  dmp_on_method_call,
+  NULL,  /* get_property */
+  NULL   /* set_property */
+};
+
+
+/* Runs in thread we created GDBusServer in (since we didn't pass G_DBUS_SERVER_FLAGS_RUN_IN_THREAD) */
+static void
+dmp_on_new_connection (GDBusServer     *server,
+                       GDBusConnection *connection,
+                       gpointer         user_data)
+{
+  DmpData *data = user_data;
+  GDBusNodeInfo *node;
+  GError *error;
+
+  /* accept the connection */
+  data->connections = g_list_prepend (data->connections, g_object_ref (connection));
+
+  error = NULL;
+  node = g_dbus_node_info_new_for_xml ("<node>"
+                                       "  <interface name='org.gtk.GDBus.DmpInterface'>"
+                                       "    <method name='AddPair'>"
+                                       "      <arg type='i' name='first' direction='in'/>"
+                                       "      <arg type='i' name='second' direction='in'/>"
+                                       "      <arg type='i' name='sum' direction='out'/>"
+                                       "    </method>"
+                                       "  </interface>"
+                                       "</node>",
+                                       &error);
+  g_assert_no_error (error);
+
+  /* sleep 100ms before exporting an object - this is to test that
+   * G_DBUS_CONNECTION_FLAGS_DELAY_MESSAGE_PROCESSING really works
+   * (GDBusServer uses this feature).
+   */
+  usleep (100 * 1000);
+
+  /* export an object */
+  error = NULL;
+  g_dbus_connection_register_object (connection,
+                                     "/dmp/test",
+                                     node->interfaces[0],
+                                     &dmp_interface_vtable,
+                                     data,
+                                     NULL,
+                                     &error);
+  g_dbus_node_info_unref (node);
+}
+
+static gpointer
+dmp_thread_func (gpointer user_data)
+{
+  DmpData *data = user_data;
+  GError *error;
+  gchar *guid;
+
+  data->context = g_main_context_new ();
+  g_main_context_push_thread_default (data->context);
+
+  error = NULL;
+  guid = g_dbus_generate_guid ();
+  data->server = g_dbus_server_new_sync ("unix:tmpdir=/tmp/gdbus-test-",
+                                         G_DBUS_SERVER_FLAGS_NONE,
+                                         guid,
+                                         NULL, /* GDBusAuthObserver */
+                                         NULL, /* GCancellable */
+                                         &error);
+  g_assert_no_error (error);
+  g_signal_connect (data->server,
+                    "new-connection",
+                    G_CALLBACK (dmp_on_new_connection),
+                    data);
+
+  g_dbus_server_start (data->server);
+
+  data->loop = g_main_loop_new (data->context, FALSE);
+  g_main_loop_run (data->loop);
+
+  g_main_context_pop_thread_default (data->context);
+
+  g_free (guid);
+  return NULL;
+}
+
+static void
+delayed_message_processing (void)
+{
+  GError *error;
+  DmpData *data;
+  GThread *service_thread;
+  guint n;
+
+  data = g_new0 (DmpData, 1);
+
+  error = NULL;
+  service_thread = g_thread_create (dmp_thread_func,
+                                    data,
+                                    TRUE,
+                                    &error);
+  while (data->server == NULL || !g_dbus_server_is_active (data->server))
+    g_thread_yield ();
+
+  for (n = 0; n < 5; n++)
+    {
+      GDBusConnection *c;
+      GVariant *res;
+      gint32 val;
+
+      error = NULL;
+      c = g_dbus_connection_new_for_address_sync (g_dbus_server_get_client_address (data->server),
+                                                  G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+                                                  NULL, /* GDBusAuthObserver */
+                                                  NULL, /* GCancellable */
+                                                  &error);
+      g_assert_no_error (error);
+
+      error = NULL;
+      res = g_dbus_connection_call_sync (c,
+                                         NULL,    /* bus name */
+                                         "/dmp/test",
+                                         "org.gtk.GDBus.DmpInterface",
+                                         "AddPair",
+                                         g_variant_new ("(ii)", 2, n),
+                                         G_VARIANT_TYPE ("(i)"),
+                                         G_DBUS_CALL_FLAGS_NONE,
+                                         -1, /* timeout_msec */
+                                         NULL, /* GCancellable */
+                                         &error);
+      g_assert_no_error (error);
+      g_variant_get (res, "(i)", &val);
+      g_assert_cmpint (val, ==, 2 + n);
+      g_variant_unref (res);
+      g_object_unref (c);
+  }
+
+  g_main_loop_quit (data->loop);
+  g_thread_join (service_thread);
+  dmp_data_free (data);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static gboolean
+nonce_tcp_on_authorize_authenticated_peer (GDBusAuthObserver *observer,
+                                           GIOStream         *stream,
+                                           GCredentials      *credentials,
+                                           gpointer           user_data)
+{
+  PeerData *data = user_data;
+  gboolean authorized;
+
+  data->num_connection_attempts++;
+
+  authorized = TRUE;
+  if (!data->accept_connection)
+    {
+      authorized = FALSE;
+      g_main_loop_quit (loop);
+    }
+
+  return authorized;
+}
+
+/* Runs in thread we created GDBusServer in (since we didn't pass G_DBUS_SERVER_FLAGS_RUN_IN_THREAD) */
+static void
+nonce_tcp_on_new_connection (GDBusServer *server,
+                             GDBusConnection *connection,
+                             gpointer user_data)
+{
+  PeerData *data = user_data;
+
+  g_ptr_array_add (data->current_connections, g_object_ref (connection));
+
+  g_main_loop_quit (loop);
+}
+
+static gpointer
+nonce_tcp_service_thread_func (gpointer user_data)
+{
+  PeerData *data = user_data;
+  GMainContext *service_context;
+  GDBusAuthObserver *observer;
+  GError *error;
+
+  service_context = g_main_context_new ();
+  g_main_context_push_thread_default (service_context);
+
+  error = NULL;
+  observer = g_dbus_auth_observer_new ();
+  server = g_dbus_server_new_sync ("nonce-tcp:",
+                                   G_DBUS_SERVER_FLAGS_NONE,
+                                   test_guid,
+                                   observer,
+                                   NULL, /* cancellable */
+                                   &error);
+  g_assert_no_error (error);
+
+  g_signal_connect (server,
+                    "new-connection",
+                    G_CALLBACK (nonce_tcp_on_new_connection),
+                    data);
+  g_signal_connect (observer,
+                    "authorize-authenticated-peer",
+                    G_CALLBACK (nonce_tcp_on_authorize_authenticated_peer),
+                    data);
+  g_object_unref (observer);
+
+  g_dbus_server_start (server);
+
+  service_loop = g_main_loop_new (service_context, FALSE);
+  g_main_loop_run (service_loop);
+
+  g_main_context_pop_thread_default (service_context);
+
+  g_main_loop_unref (service_loop);
+  g_main_context_unref (service_context);
+
+  /* test code specifically unrefs the server - see below */
+  g_assert (server == NULL);
+
+  return NULL;
+}
+
+static void
+test_nonce_tcp (void)
+{
+  PeerData data;
+  GError *error;
+  GThread *service_thread;
+  GDBusConnection *c;
+  gchar *s;
+  gchar *nonce_file;
+  gboolean res;
+  const gchar *address;
+
+  memset (&data, '\0', sizeof (PeerData));
+  data.current_connections = g_ptr_array_new_with_free_func (g_object_unref);
+
+  error = NULL;
+  server = NULL;
+  service_loop = NULL;
+  service_thread = g_thread_create (nonce_tcp_service_thread_func,
+                                    &data,
+                                    TRUE,
+                                    &error);
+  while (service_loop == NULL)
+    g_thread_yield ();
+  g_assert (server != NULL);
+
+
+  /* bring up a connection and accept it */
+  data.accept_connection = TRUE;
+  error = NULL;
+  c = g_dbus_connection_new_for_address_sync (g_dbus_server_get_client_address (server),
+                                              G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+                                              NULL, /* GDBusAuthObserver */
+                                              NULL, /* cancellable */
+                                              &error);
+  g_assert_no_error (error);
+  g_assert (c != NULL);
+  while (data.current_connections->len < 1)
+    g_main_loop_run (loop);
+  g_assert_cmpint (data.current_connections->len, ==, 1);
+  g_assert_cmpint (data.num_connection_attempts, ==, 1);
+  g_assert (g_dbus_connection_get_unique_name (c) == NULL);
+  g_assert_cmpstr (g_dbus_connection_get_guid (c), ==, test_guid);
+  g_object_unref (c);
+
+  /* now, try to subvert the nonce file (this assumes noncefile is the last key/value pair)
+   */
+
+  address = g_dbus_server_get_client_address (server);
+
+  s = strstr (address, "noncefile=");
+  g_assert (s != NULL);
+  s += sizeof "noncefile=" - 1;
+  nonce_file = g_strdup (s);
+
+  /* First try invalid data in the nonce file - this will actually
+   * make the client send this and the server will reject it. The way
+   * it works is that if the nonce doesn't match, the server will
+   * simply close the connection. So, from the client point of view,
+   * we can see a variety of errors.
+   */
+  error = NULL;
+  res = g_file_set_contents (nonce_file,
+                             "0123456789012345",
+                             -1,
+                             &error);
+  g_assert_no_error (error);
+  g_assert (res);
+  c = g_dbus_connection_new_for_address_sync (address,
+                                              G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+                                              NULL, /* GDBusAuthObserver */
+                                              NULL, /* cancellable */
+                                              &error);
+  _g_assert_error_domain (error, G_IO_ERROR);
+  g_assert (c == NULL);
+
+  /* Then try with a nonce-file of incorrect length - this will make
+   * the client complain - we won't even try connecting to the server
+   * for this
+   */
+  error = NULL;
+  res = g_file_set_contents (nonce_file,
+                             "0123456789012345_",
+                             -1,
+                             &error);
+  g_assert_no_error (error);
+  g_assert (res);
+  c = g_dbus_connection_new_for_address_sync (address,
+                                              G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+                                              NULL, /* GDBusAuthObserver */
+                                              NULL, /* cancellable */
+                                              &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert (c == NULL);
+
+  /* Finally try with no nonce-file at all */
+  g_assert_cmpint (g_unlink (nonce_file), ==, 0);
+  error = NULL;
+  c = g_dbus_connection_new_for_address_sync (address,
+                                              G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+                                              NULL, /* GDBusAuthObserver */
+                                              NULL, /* cancellable */
+                                              &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT);
+  g_assert (c == NULL);
+
+  g_free (nonce_file);
+
+  g_dbus_server_stop (server);
+  g_object_unref (server);
+  server = NULL;
+
+  g_main_loop_quit (service_loop);
+  g_thread_join (service_thread);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 int
 main (int   argc,
       char *argv[])
@@ -753,6 +1241,8 @@ main (int   argc,
   loop = g_main_loop_new (NULL, FALSE);
 
   g_test_add_func ("/gdbus/peer-to-peer", test_peer);
+  g_test_add_func ("/gdbus/delayed-message-processing", delayed_message_processing);
+  g_test_add_func ("/gdbus/nonce-tcp", test_nonce_tcp);
 
   ret = g_test_run();
 
