@@ -405,6 +405,18 @@ dbus_1_message_append (GString *s,
         break;
       }
 
+#ifdef DBUS_TYPE_UNIX_FD
+    case DBUS_TYPE_UNIX_FD:
+      {
+        /* unfortunately there's currently no way to get just the
+         * protocol value, since dbus_message_iter_get_basic() wants
+         * to be 'helpful' and dup the fd for the user...
+         */
+        g_string_append (s, "unix-fd: (not extracted)\n");
+        break;
+      }
+#endif
+
      case DBUS_TYPE_VARIANT:
        g_string_append_printf (s, "variant:\n");
        dbus_message_iter_recurse (iter, &sub);
@@ -484,12 +496,19 @@ get_body_signature (GVariant *value)
   gsize len;
   gchar *ret;
 
+  if (value == NULL)
+    {
+      ret = g_strdup ("");
+      goto out;
+    }
+
   s = g_variant_get_type_string (value);
   len = strlen (s);
-  g_assert (len>=2);
+  g_assert (len >= 2);
 
   ret = g_strndup (s + 1, len - 2);
 
+ out:
   return ret;
 }
 
@@ -560,20 +579,27 @@ check_serialization (GVariant *value,
                                                     blob_size,
                                                     G_DBUS_CAPABILITY_FLAGS_NONE,
                                                     &error);
-  g_assert_no_error (error);
   g_assert (recovered_message != NULL);
-  g_assert (g_dbus_message_get_body (recovered_message) != NULL);
+  g_assert_no_error (error);
 
-  if (!g_variant_equal (g_dbus_message_get_body (recovered_message), value))
+  if (value == NULL)
     {
-      s = g_variant_print (g_dbus_message_get_body (recovered_message), TRUE);
-      s1 = g_variant_print (value, TRUE);
-      g_printerr ("Recovered value:\n%s\ndoes not match given value\n%s\n",
-                  s,
-                  s1);
-      g_free (s);
-      g_free (s1);
-      g_assert_not_reached ();
+      g_assert (g_dbus_message_get_body (recovered_message) == NULL);
+    }
+  else
+    {
+      g_assert (g_dbus_message_get_body (recovered_message) != NULL);
+      if (!g_variant_equal (g_dbus_message_get_body (recovered_message), value))
+        {
+          s = g_variant_print (g_dbus_message_get_body (recovered_message), TRUE);
+          s1 = g_variant_print (value, TRUE);
+          g_printerr ("Recovered value:\n%s\ndoes not match given value\n%s\n",
+                      s,
+                      s1);
+          g_free (s);
+          g_free (s1);
+          g_assert_not_reached ();
+        }
     }
   g_object_unref (message);
   g_object_unref (recovered_message);
@@ -582,6 +608,8 @@ check_serialization (GVariant *value,
 static void
 message_serialize_basic (void)
 {
+  check_serialization (NULL, "");
+
   check_serialization (g_variant_new ("(sogybnqiuxtd)",
                                       "this is a string",
                                       "/this/is/a/path",
@@ -647,6 +675,31 @@ message_serialize_complex (void)
                        "value 1:   array:\n"
                        "value 2:   array:\n"
                        "    string: `Something'\n");
+
+  /* https://bugzilla.gnome.org/show_bug.cgi?id=621838 */
+  check_serialization (g_variant_new_parsed ("(@aay [], {'cwd': <'/home/davidz/Hacking/glib/gio/tests'>})"),
+                       "value 0:   array:\n"
+                       "value 1:   array:\n"
+                       "    dict_entry:\n"
+                       "      string: `cwd'\n"
+                       "      variant:\n"
+                       "        string: `/home/davidz/Hacking/glib/gio/tests'\n");
+
+#ifdef DBUS_TYPE_UNIX_FD
+  value = g_variant_parse (G_VARIANT_TYPE ("(hah)"),
+                           "(42, [43, 44])",
+                           NULL, NULL, &error);
+  g_assert_no_error (error);
+  g_assert (value != NULL);
+  /* about (not extracted), see comment in DBUS_TYPE_UNIX_FD case in
+   * dbus_1_message_append() above.
+   */
+  check_serialization (value,
+                       "value 0:   unix-fd: (not extracted)\n"
+                       "value 1:   array:\n"
+                       "    unix-fd: (not extracted)\n"
+                       "    unix-fd: (not extracted)\n");
+#endif
 }
 
 
