@@ -112,7 +112,6 @@
 
 #include "glibintl.h"
 
-#include "gobjectalias.h"
 
 GType
 g_binding_flags_get_type (void)
@@ -333,10 +332,7 @@ on_source_notify (GObject    *gobject,
   if (binding->is_frozen)
     return;
 
-  if (pspec->flags & G_PARAM_STATIC_NAME)
-    p_name = g_intern_static_string (pspec->name);
-  else
-    p_name = g_intern_string (pspec->name);
+  p_name = g_intern_string (pspec->name);
 
   if (p_name != binding->source_property)
     return;
@@ -377,10 +373,7 @@ on_target_notify (GObject    *gobject,
   if (binding->is_frozen)
     return;
 
-  if (pspec->flags & G_PARAM_STATIC_NAME)
-    p_name = g_intern_static_string (pspec->name);
-  else
-    p_name = g_intern_string (pspec->name);
+  p_name = g_intern_string (pspec->name);
 
   if (p_name != binding->target_property)
     return;
@@ -958,6 +951,3 @@ g_object_bind_property (gpointer       source,
                                       NULL,
                                       NULL, NULL);
 }
-
-#define __G_BINDING_C__
-#include "gobjectaliasdef.c"
